import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.io.InputStream;
import java.util.Timer;
import java.util.TimerTask;

import javax.imageio.ImageIO;
import javax.swing.JFrame;
import javax.swing.JPanel;


public class Microwave extends JPanel {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	final BufferedImage microwaveOff;
	final BufferedImage microwaveOn;
	BufferedImage currImage;
	final AffineTransform transform;
	double scale;
	final Timer t = new Timer(true);
	int time;
	CountDown countDown;

	public Microwave()
	{
		InputStream is = Microwave.class.getResourceAsStream(
				  "microwave-off.png");
		try {
			currImage = microwaveOff = ImageIO.read(is);
		} catch (java.io.IOException ex) {
			throw new RuntimeException(ex);
		}
		is = Microwave.class.getResourceAsStream("microwave-on.png");
		try {
			microwaveOn = ImageIO.read(is);
		} catch (java.io.IOException ex) {
			throw new RuntimeException(ex);
		}
		transform = new AffineTransform();
		transform.translate(0, 0);
		transform.scale(1, 1);
	}
	private void updateScale () {
		scale = Math.min((double)getWidth() / currImage.getWidth(),
			(double)getHeight() / currImage.getHeight());
	}
	
	public int getTime() { return time; }
	public void setTime(int ti) {
		time = ++ti;
		if (countDown == null) {
	        turnOn();
	        t.schedule(countDown = new CountDown(), 0, 1000);
	    }
	}
	class CountDown extends TimerTask {
        public void run() {
            time -= 1;
            if (time == 0)
            {
                countDown = null;
                this.cancel();
                inform();
            }
        }
    }
	public void cancelTimer()
	{
		turnOff();
		countDown = null;
		countDown.cancel();
	}
	protected void inform()
	{
		turnOff();
	}
	protected void turnOn() {
		if (currImage == microwaveOn)
			return;
		currImage = microwaveOn;
		repaint();
	}
	protected void turnOff() {
		if (currImage == microwaveOff)
			return;
		currImage = microwaveOff;
		repaint();
	}
	public void paintComponent(Graphics g)
	{
		super.paintComponent(g);
		updateScale();
		Graphics2D g2d = (Graphics2D)g;
		g2d.scale(scale, scale);
		g2d.drawRenderedImage(currImage, transform);
	}

	public static void main(String [] args)
	{
		JFrame jf = new JFrame();
		Microwave mpanel = new Microwave();
		jf.getContentPane().add(mpanel);
		jf.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		jf.setSize(400, 300);
		jf.setTitle("Microwave");
		jf.setVisible(true);
		
		mpanel.setTime(5);
	}
}
