package org.ersoft.ooad.hacs;

import java.util.*;
import org.freedesktop.Avahi.*;
import org.freedesktop.dbus.*;

import org.snmp4j.Snmp;
import org.snmp4j.smi.UdpAddress;
import org.snmp4j.transport.DefaultUdpTransportMapping;

// Exceptions
import org.freedesktop.dbus.exceptions.DBusException;
import java.io.IOException;

public class Controller {
	final DBusConnection bus;
	final Server avahi;
	ServiceBrowser browser;
	ServiceResolver resolver;
	
	final Snmp snmp;

	final Failure failure = new Failure();
	final ItemNew itemNew = new ItemNew();
	final ItemRemove itemRemove = new ItemRemove();

	Map<String,Device> devices = new TreeMap<String,Device>();

	public Controller() throws DBusException, IOException {
		int port = new Random().nextInt(64000) + 1024;
		snmp = new Snmp(new DefaultUdpTransportMapping(new UdpAddress(port)));
		snmp.listen();

		bus = DBusConnection.getConnection(DBusConnection.SYSTEM);
		bus.addSigHandler(ServiceBrowser.Failure.class, failure);
		bus.addSigHandler(ServiceBrowser.ItemNew.class, itemNew);
		bus.addSigHandler(ServiceBrowser.ItemRemove.class, itemRemove);

		avahi = (Server)bus.getRemoteObject(
			"org.freedesktop.Avahi", "/", Server.class);

		browser = (ServiceBrowser)avahi.ServiceBrowserNew(
			-1, -1, "_hacs._udp", "", new UInt32(0));
	}
	public void disconnect() throws DBusException, IOException {
		//browser.Free();
		snmp.close();
		bus.removeSigHandler(ServiceBrowser.Failure.class, failure);
		bus.removeSigHandler(ServiceBrowser.ItemNew.class, itemNew);
		bus.removeSigHandler(ServiceBrowser.ItemRemove.class, itemRemove);
		bus.disconnect();
	}

	class Failure implements DBusSigHandler<ServiceBrowser.Failure> {
		public void handle(ServiceBrowser.Failure sig) {
			throw new RuntimeException("Failure in Service Browser: " + sig.error);
		}
	}
	class ItemNew implements DBusSigHandler<ServiceBrowser.ItemNew> {
		public void handle(ServiceBrowser.ItemNew sig) {
			devices.put(sig.name, new Device(Controller.this, sig.iinterface, sig.protocol, sig.name, sig.type, sig.domain));
		}
	}
	class ItemRemove implements DBusSigHandler<ServiceBrowser.ItemRemove> {
		public void handle(ServiceBrowser.ItemRemove sig) {
			Device dev = devices.get(sig.name);
			if (dev == null)
				return;
			if (!dev.equals(new Device(Controller.this, sig.iinterface, sig.protocol, sig.name, sig.type, sig.domain)))
				return;
			devices.remove(sig.name);
		}
	}

	public Device[] getDevices() {
		return devices.values().toArray(new Device[0]);
	}
	public Device getDevice(String name) {
		return devices.get(name);
	}
	public void addDeviceListener(DeviceListener listener) {}
	public void removeDeviceListener(DeviceListener listener) {}
	public Snmp getSnmp() { return snmp; }
	public Server getAvahi() { return avahi; }
}
