package org.ersoft.ooad.hacs;

import java.util.*;
import java.io.IOException;
import org.freedesktop.Avahi.*;
import org.freedesktop.dbus.*;

import org.snmp4j.*;
import org.snmp4j.smi.*;
import org.snmp4j.mp.SnmpConstants;
import org.snmp4j.event.ResponseEvent;
import java.net.InetAddress;

public class Device implements Comparable<Device> {
	final Controller controller;
	CommunityTarget target;
	final int iinterface, protocol;
	final String name, type, domain;
	Map<String,String> metadata;
	Map<String,Property> props;

	public Device(Controller controller, int iinterface, int protocol, String name, String type, String domain) {
		this.controller = controller;
		this.iinterface = iinterface;
		this.protocol = protocol;
		this.name = name;
		this.type = type;
		this.domain = domain;
	}
	public Map<String,Property> getProperties() throws IOException {
		if (this.props != null)
			return this.props;
		// creating PDU
		Map<String,Property> props = new TreeMap<String,Property>();
		ResponseEvent e;
		OID name = new OID(new int[]{1,0,0});
		OID oname = name;
		OID value = new OID(new int[]{1,0,1});
		do {
			PDU pdu = new PDU();
			pdu.add(new VariableBinding(name));
			pdu.add(new VariableBinding(value));
			// sending request
			e = controller.getSnmp().getNext(pdu, getTarget());
			pdu = e.getResponse();
			if (pdu == null)
				throw new IOException("No response");
			name = pdu.get(0).getOid();
			value = pdu.get(1).getOid();
			if (!name.startsWith(oname))
				break;
			Variable nameVar = pdu.get(0).getVariable();
			if (!(nameVar instanceof OctetString))
				break;
				// TODO: figure out what is going wrong here with the microwave
				//throw new IOException("Proprety name not OctetString, it is " + nameVar.getClass());
			String strName = nameVar.toString();
			Property p = Property.create(this, value, strName, pdu.get(1).getVariable());
			props.put(strName, p);
		} while (e.getResponse() != null);
		this.props = props;
		return props;
	}
	public Map<String,String> getMetadata() {
		if (metadata != null)
			return metadata;
		NTuple11<Integer, Integer, String, String, String, String, Integer, String, UInt16, List<List<Byte>>, UInt32> retTuple;
		retTuple = controller.getAvahi().ResolveService(iinterface, protocol, name, type, domain, -1, new UInt32(0));
		metadata = new HashMap<String,String>();
		metadata.put("name", retTuple.c);
		metadata.put("host", retTuple.f);
		metadata.put("address", retTuple.h);
		metadata.put("port", retTuple.i.toString());
		for (List<Byte> l: retTuple.j)
		{
			String item = byteListToString(l);
			int index = item.indexOf('=');
			if (index == -1)
				metadata.put(item, null);
			else
				metadata.put(item.substring(0, index), item.substring(index+1));
		}
		return metadata;
	}
	protected String byteListToString(List<Byte> list) {
		byte[] field = new byte[list.size()];
		for (int x = 0; x < list.size(); x++)
			field[x] = list.get(x);
		try {
			return new String(field, "utf-8");
		} catch (java.io.UnsupportedEncodingException ex) {
			throw new RuntimeException(ex);
		}
	}
	public void addDeviceListener(DeviceListener listener) {}
	public void removeDeviceListener(DeviceListener listener) {}
	public void updateProperties() {
		props = null;
	}
	public int compareTo(Device dev) {
		if (iinterface != dev.iinterface)
			return iinterface - dev.iinterface;
		if (protocol != dev.protocol)
			return protocol - dev.protocol;
		int comp;
		comp = name.compareTo(dev.name);
		if (comp != 0)
			return comp;
		comp = type.compareTo(dev.type);
		if (comp != 0)
			return comp;
		comp = domain.compareTo(dev.domain);
		return comp;
	}
	public boolean equals(Object obj) {
		if (!(obj instanceof Device)) return false;
		return compareTo((Device)obj) == 0;
	}
	public int hashCode() {
		int hash = name.hashCode() ^ type.hashCode() ^ domain.hashCode();
		hash ^= iinterface ^ protocol;
		return hash;
	}
	public String toString() {
		return name;
	}

	public String getName() { return name; }
	public Controller getController() { return controller; }

	public CommunityTarget getTarget() throws IOException {
		if (target != null)
			return target;
		// setting up target
		CommunityTarget target = new CommunityTarget();
		target.setCommunity(new OctetString("public"));
		InetAddress remoteHost = InetAddress.getByName(getMetadata().get("address"));
		int port = Integer.parseInt(getMetadata().get("port"));
		target.setAddress(new UdpAddress(remoteHost, port));
		target.setRetries(1);
		target.setTimeout(1000);
		target.setVersion(SnmpConstants.version1);
		this.target = target;
		return target;
	}
}
