package org.ersoft.ooad.hacs;

import java.io.IOException;
import org.ersoft.ooad.hacs.property.*;
import org.snmp4j.PDU;
import org.snmp4j.smi.*;
import org.snmp4j.event.ResponseEvent;

public abstract class Property<T,U extends Variable> {
	protected final Device device;
	protected final OID oidValue;
	protected final String name;
	protected T obj;
	
	public Property(Device device, OID oidValue, String name, T obj) {
		this.device = device;
		this.oidValue = oidValue;
		this.name = name;
		this.obj = obj;
	}
	public Property(Device device, OID oidValue, String name, U obj) {
		this.device = device;
		this.oidValue = oidValue;
		this.name = name;
		this.obj = fromVariable(obj);
	}
	public String getName() { return name; }
	public T getValue() { return obj; }
	public boolean setValue(T value) throws IOException {
		ResponseEvent re;
		Variable v = toVariable(value);
		PDU pdu = new PDU();
		pdu.add(new VariableBinding(oidValue, v));
		re = device.getController().getSnmp().set(pdu, device.getTarget());
		if (re.getResponse() == null)
			throw new IOException("No response");
		v = re.getResponse().get(0).getVariable();
		obj = fromVariable((U)v);
		return true;
	}
	public void setValue(T value, int pin) throws IOException {
		throw new RuntimeException("not implemented");
	}
	public boolean updateValue() throws IOException {
		return false;
	}
	public void setProtection(boolean enable, int pin) throws IOException {
		throw new RuntimeException("not implemented");
	}
	public void addDeviceListener(DeviceListener listener) {}
	public void removeDeviceListener(DeviceListener listener) {}
	protected abstract U toVariable(T o);
	protected abstract T fromVariable(U v);
	public static Property create(Device device, OID oidValue, String name, Variable obj) {
		if (obj instanceof OctetString) {
			return new StringProperty(device, oidValue, name, (OctetString)obj);
		}
		return null;
	}
}
