package org.ersoft.ooad.servlet;

import java.io.*;
import java.util.*;
import java.util.regex.*;
import javax.servlet.*;
import javax.servlet.http.*;
import org.apache.velocity.app.VelocityEngine;
import org.apache.velocity.*;
import org.ersoft.ooad.hacs.*;

public class HACSServlet extends HttpServlet {
	Controller controller;
	List<Service> services;
	VelocityEngine ve;
	public void init() throws ServletException {
		try {
			controller = new Controller();
		} catch (Exception ex) {
			throw new ServletException(ex);
		}
		ve = new VelocityEngine();
		Properties prop = new Properties();
		prop.setProperty("file.resource.loader.path", getServletContext().getRealPath("/templates"));
		prop.setProperty("file.resource.loader.cache", "true");
		try {
			ve.init(prop);
		} catch (Exception ex) {
			throw new ServletException(ex);
		}
		services = new ArrayList<Service>();
		services.add(new StaticContent());
		services.add(new LogoutService());
		services.add(new Home());
		services.add(new Sort());
		services.add(new DeviceProps());
		services.add(new NotFound());
	}
	public void destroy() {
		try {
			controller.disconnect();
		} catch (Exception ex) {
			throw new RuntimeException(ex);
		}
		controller = null;
	}
	public void doGet(HttpServletRequest request, HttpServletResponse response)
			throws IOException, ServletException {
		String path = request.getServletPath();
		for (Service s: services)
		{
			Matcher m = s.servicePath.matcher(path);
			if (!m.matches())
				continue;
			int count = m.groupCount();
			String[] groups = new String[count + 1];
			for (int x = 0; x < groups.length; x++)
				groups[x] = m.group(x);
			s.service(request, response, groups);
			break;
		}
	}
	public void doPost(HttpServletRequest request, HttpServletResponse response)
			throws IOException, ServletException {
		doGet(request, response);
	}

	/* Static Content Service */
	class StaticContent extends Service {
		RequestDispatcher staticContent;
		public StaticContent() {
			super("/shared/.*");
			staticContent = getServletContext().getNamedDispatcher("StaticContent");
		}
		public void service(HttpServletRequest request,
				HttpServletResponse response, String[] groups)
				throws IOException, ServletException {
			staticContent.forward(request, response);
		}
	}

class LogoutService extends Service {
	public LogoutService() { super("/logout"); }
	public void service(HttpServletRequest request,
			HttpServletResponse response, String[] groups)
			throws IOException, ServletException {
		Cookie c = new Cookie("username", "");
		c.setMaxAge(0);
		response.addCookie(c);
		c = new Cookie("password", "");
		c.setMaxAge(0);
		response.addCookie(c);
		response.sendRedirect(request.getContextPath());
	}
}
/* Template Services */
abstract class TemplateService extends Service {
	Template template;
	public TemplateService(String regex, String template) {
		super(regex);
		try {
			this.template = ve.getTemplate(template);
		} catch (Exception ex) {
			throw new RuntimeException(ex);
		}
	}
	public void service(HttpServletRequest request,
			HttpServletResponse response, String[] groups)
			throws IOException, ServletException {
		VelocityContext context = new VelocityContext();
		context.put("contextPath", request.getContextPath());
		response.setContentType("text/html; charset=UTF-8");
		if (service(request, response, groups, context))
			template.merge(context, response.getWriter());
	}
	abstract boolean service(HttpServletRequest request,
			HttpServletResponse response, String[] groups, VelocityContext ctx);
}
class LoginService extends TemplateService {
	public LoginService() { super("", "login.vm"); }
	public boolean service(HttpServletRequest request,
			HttpServletResponse response, String[] groups, VelocityContext ctx) {
		String username = null, password = null;
		{
			Map map = request.getParameterMap();
			String[] array;
			array = (String[])map.get("username");
			if (array != null && array.length > 0)
				username = array[0];
			array = (String[])map.get("password");
			if (array != null && array.length > 0)
				password = array[0];
		}
		if (username == null || password == null)
			return true;
		if (username.equals("")) {
			ctx.put("error", "Invalid username or password");
			return true;
		}
		if (!username.equals("admin") || !password.equals("admin"))
		{
			ctx.put("error", "Invalid username or password");
			return true;
		}
		response.addCookie(new Cookie("username", username));
		response.addCookie(new Cookie("password", password));
		try {
			response.sendRedirect(request.getRequestURI());
		} catch (IOException ex) {
			throw new RuntimeException(ex);
		}
		return false;
	}
	public User authenticate(HttpServletRequest request) {
		String username = null, password = null;
		Cookie[] cs = request.getCookies();
		if (cs == null)
			return null;
		for (Cookie c: cs) {
			String name = c.getName();
			if (name.equals("username"))
				username = c.getValue();
			else if (name.equals("password"))
				password = c.getValue();
		}
		if (username.equals("admin") && password.equals("admin"))
			return new User();
		else
			return null;
	}
}
abstract class AuthorizedService extends TemplateService {
	LoginService loginService = new LoginService();
	public AuthorizedService(String regex, String template) {
		super(regex, template);
	}
	public boolean service(HttpServletRequest request,
			HttpServletResponse response, String[] groups, VelocityContext ctx) {
		User user = loginService.authenticate(request);
		if (user == null) {
			try {
				loginService.service(request, response, groups);
			} catch (Exception ex) {
				throw new RuntimeException(ex);
			}
			return false;
		}
		ctx.put("user", user);
		String sortType = "room";
		{
			Map map = request.getParameterMap();
			String[] array;
			array = (String[])map.get("sort");
			if (array != null && array.length > 0) {
				response.addCookie(new Cookie("sort",sortType));
				sortType = array[0];
			}
		}
		
		
		for ( Cookie c : request.getCookies() )
			if (c.getName().equals("sort") )
				sortType = c.getValue();
		Map<String, List<Device>> categories = new TreeMap<String, List<Device>>();
		for(Device device: controller.getDevices()) {
			String temp = device.getMetadata().get(sortType);
			List<Device> c = categories.get(temp);
			if (c == null) {
				List<Device> l = new ArrayList<Device>();
				l.add(device);
				categories.put(temp, l);
			} else {
				c.add(device);
			}
		}
		ctx.put("sort", categories);
		ctx.put("SortType", sortType);
		ctx.put("devices", controller.getDevices());
		return service(request, response, groups, ctx, user);
	}
	public abstract boolean service(HttpServletRequest request,
			HttpServletResponse response, String[] groups, VelocityContext ctx,
			User user);
}
class Home extends AuthorizedService {
	AuthorizedService sort = new Sort();
	public Home() { super("/", "home.vm"); }
	public boolean service(HttpServletRequest request,
			HttpServletResponse response, String[] groups, VelocityContext ctx,
			User user) {
		String[] tmp = new String[2];
		tmp[0] = groups[0];
		tmp[1] = "room";
		return sort.service(request, response, tmp, ctx, user);
	}
}
class Sort extends AuthorizedService {
	public Sort() { super("/sort/([^/]+)", "home.vm"); }
	public boolean service(HttpServletRequest request,
			HttpServletResponse response, String[] groups, VelocityContext ctx,
			User user) {
		
		
		return true;
	}
}
class DeviceProps extends AuthorizedService {
	public DeviceProps() { super("/device/([^/]+)/?", "device.vm"); }
	public boolean service(HttpServletRequest request,
			HttpServletResponse response, String[] groups, VelocityContext ctx,
			User user) {
		Device d = controller.getDevice(groups[1]);
		if (d == null)
			return true;
		List<String> errors = new ArrayList<String>();
		List<String> msgs = new ArrayList<String>();
		ctx.put("device", d);
		Map map = request.getParameterMap();
		for (Object o: map.keySet()) {
			String key = (String)o;
			String val = ((String[])map.get(o))[0];
			Property prop;
			try {
			prop = d.getProperties().get(key);
			} catch (IOException ex) {
				errors.add(ex.getMessage());
				continue;
			}
			if (prop == null)
				errors.add("No property with name '" + key + "'");
			if (prop.getValue().equals(val))
				continue;
			try {
			if (prop.setValue(val))
				msgs.add("'" + key + "' set to '" + val + "'");
			else
				errors.add("Something went wrong");
			} catch (IOException ex) {
				errors.add(ex.getMessage());
			}
		}
		try {
			ctx.put("properties", d.getProperties());
		} catch (IOException ex) {
			errors.add(ex.getMessage());
			ex.printStackTrace();
		}
		ctx.put("error", errors);
		ctx.put("messages", msgs);
		return true;
	}
}
class NotFound extends TemplateService {
	public NotFound() { super(".*", "404.vm"); }
	public boolean service(HttpServletRequest request,
			HttpServletResponse response, String[] groups, VelocityContext ctx) {
		response.setStatus(response.SC_NOT_FOUND);
		ctx.put("devices", controller.getDevices());
		return true;
	}
}

}

abstract class Service {
	public final Pattern servicePath;
	public Service(String regex) {
		servicePath = Pattern.compile(regex);
	}
	public abstract void service(HttpServletRequest request,
		HttpServletResponse response, String[] groups)
		throws IOException, ServletException;
}

class User {}
